<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Models\AttendanceRecord;
use Carbon\Carbon;

class AttendanceController extends Controller
{
    public function store(Request $request)
    {
        $request->validate([
            'student_id' => 'required|integer',
        ]);

        $studentId = $request->get('student_id');
        $timestamp = Carbon::now()->toDateTimeString();

        // Check the last record for the student
        $lastRecord = AttendanceRecord::where('student_id', $studentId)
            ->orderBy('timestamp', 'desc')
            ->first();

        // Determine if the new record is "in" or "out"
        $attendance = 'in';
        if ($lastRecord && $lastRecord->attendance === 'in') {
            $attendance = 'out';
        }

        $attendanceRecord = new AttendanceRecord([
            'student_id' => $studentId,
            'attendance' => $attendance,
            'timestamp' => $timestamp,
        ]);

        $attendanceRecord->save();

        return response()->json('Attendance record saved successfully');
    }

    public function getAttendanceByStudent($student_id)
    {
        $attendanceRecords = AttendanceRecord::where('student_id', $student_id)->get();

        return response()->json($attendanceRecords);
    }

    public function getAllAttendance()
    {
        $attendanceRecords = DB::table('reader_records as ar')
        ->join('students as s', 'ar.student_id', '=', 's.idno')
        ->select('s.name', 'ar.attendance', 'ar.timestamp')
        ->orderBy('ar.timestamp')
        ->get();

        $formattedRecords = [];
        $entry = null;

        foreach ($attendanceRecords as $record) {
            if ($record->attendance == 'in') {
                $entry = $record->timestamp;
            } elseif ($record->attendance == 'out' && $entry) {
                $exit = $record->timestamp;
                $entryTime = Carbon::parse($entry);
                $exitTime = Carbon::parse($exit);
                $durationInMinutes = $entryTime->diffInMinutes($exitTime);

                if ($durationInMinutes >= 60) {
                    $hours = floor($durationInMinutes / 60);
                    $minutes = $durationInMinutes % 60;
                    $duration = $hours . ' hours ' . ($minutes > 0 ? $minutes . ' minutes' : '');
                } else {
                    $duration = floor($durationInMinutes) . ' minutes';
                }

                $formattedRecords[] = [
                    'name' => $record->name,
                    'entry' => $entryTime->format('d/m/Y H:i:s'),
                    'exit' => $exitTime->format('d/m/Y H:i:s'),
                    'duration' => $duration,
                ];

                $entry = null; // Reset entry for the next pair
            }
        }

        return response()->json($formattedRecords);
    }

    public function getAttendanceStatus()
    {
        $students = DB::table('students as s')
            ->leftJoin('reader_records as ar', 's.idno', '=', 'ar.student_id')
            ->select('s.name', DB::raw('MAX(ar.timestamp) as last_activity'))
            ->groupBy('s.id', 's.name')
            ->get();

        $formattedRecords = [];

        foreach ($students as $student) {
            $status = 'Absent';
            $date = null;
            if ($student->last_activity) {
                $lastActivity = Carbon::parse($student->last_activity);
                $date = $lastActivity->format('d/m/Y');
                if ($lastActivity->isToday()) {
                    $status = 'Present';
                }
            }

            $formattedRecords[] = [
                'name' => $student->name,
                'status' => $status,
                'date' => $date,
            ];
        }

        return response()->json($formattedRecords);
    }
}
